#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>

#include <signal.h>

#define osErrorFatal(userMsg) osErrorFatalImpl((userMsg), __FILE__, __func__, __LINE__)
#define osAssert(expr, userMsg) \
	do {\
		if (!(expr)) \
			osErrorFatal(userMsg); \
	} while (0)
	
void osErrorFatalImpl(const char* userMsg, const char* fileName, const char* functionName, const int lineNumber);

bool osShouldTerminate = false;

void osHandleSIGINT(int signum);
void osHandleSIGTERM(int signum);
void HandleSig(int signum);

int main(int argc, char** argv) {
	
	/* moguce je postaviti razlicit signal handler za svaki
	 * signal ponaosob
	 */
	//osAssert(signal(SIGINT, osHandleSIGINT) != SIG_ERR, "SIGINT handler setup failed");
	//osAssert(signal(SIGTERM, osHandleSIGTERM) != SIG_ERR, "SIGTERM handler setup failed");
	
	/* ali, moguce je i da isti handler obradjuje vise signala 
	 * 
	 * Za zainteresovane:
	 * za detaljniju kontrolu nad signalima pogledati deo u TLPI o
	 * funkciji sigaction i maskama signala 
	 */ 
	osAssert(signal(SIGINT, HandleSig) != SIG_ERR, "SIGINT handler setup failed");
	osAssert(signal(SIGTERM, HandleSig) != SIG_ERR, "SIGTERM handler setup failed");
	
	/* stampamo pid procesa da bismo znali kome da saljemo signale
	 * 
	 * signale iz komandne linije saljemo alatkom kill
	 * listanje svih signala
	 * kill -l
	 * slanje signala 
	 * kill ime_signala pid
	 * 
	 * BITNO:
	 * ekvivalentan sistemski poziv postoji i u C-u
	 * man 2 kill
	 */ 
	printf("PID is: %jd\n",(intmax_t)getpid());
	
	do  {
		
		printf("waiting fo signal\n");
		
		/* pause se koristi za cekanje na signal
		 * BITNO:
		 * cekanje sa pause() nije aktivno.
		 */ 
		pause();
		printf("signal caught\n");
		
	} while(!osShouldTerminate);
	
	exit(EXIT_SUCCESS);
}

void HandleSig(int signum) {
	
	/* argument funkcije je redni broj signala
	 * i moze se koristiti za otkrivanje o kom signalu je rec
	 */
	 
	 /* BITNO:
	  * u signal handlerima moraju da se koriste funkcije koju
	  * su reentrant safe, tj. one koje garantuju korektan nastavak
	  * izvrsavanja nakon prekida
	  */
	  
	 /* napisani primer koristi printf koji nije reentrant safe
	  * i sluzi samo kao ilustracija koncepta ali se u realnim aplikacijama
	  * nikada ne koristi nesto sto nije reentrant safe
	  */ 
	switch (signum) {
		
		case SIGINT:
			printf("SIGINT caught\n");
			break;
		case SIGTERM:
			printf("SIGTERM caught\n");
			osShouldTerminate = true;
			break;
		default:
			break;
	}
}

void osHandleSIGINT(int signum) {
	
	printf("SIGINT caught\n");
}

void osHandleSIGTERM(int signum) {
	
	printf("SIGTERM caught\n");
	osShouldTerminate = true;
}

void osErrorFatalImpl(const char* userMsg, const char* fileName, const char* functionName, const int lineNumber) {
	
	perror(userMsg);
	
	fprintf(stderr, "File name: %s\nFunction name: %s\nLine number: %d\n", fileName, functionName, lineNumber);
	
	exit(EXIT_FAILURE);
}
